
import threading
import logging
import io
import wave
import numpy as np
import tempfile
import os
import websocket
import base64
import hashlib
import hmac
import json
from datetime import datetime
from time import mktime
from wsgiref.handlers import format_date_time
import ssl

from urllib.parse import urlencode
import _thread as thread



logger = logging.getLogger(__name__)

# 讯飞 TTS 参数
class Ws_Param:
    def __init__(self, APPID, APIKey, APISecret, Text):
        self.APPID = APPID
        self.APIKey = APIKey
        self.APISecret = APISecret
        self.Text = Text
        self.CommonArgs = {"app_id": self.APPID}
        self.BusinessArgs = {"aue": "raw", "auf": "audio/L16;rate=16000", "vcn": "xiaoyan", "tte": "utf8"}
        self.Data = {"status": 2, "text": str(base64.b64encode(self.Text.encode('utf-8')), "UTF8")}

    def create_url(self):
        url = 'wss://tts-api.xfyun.cn/v2/tts'
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))
        signature_origin = "host: " + "ws-api.xfyun.cn" + "\n"
        signature_origin += "date: " + date + "\n"
        signature_origin += "GET " + "/v2/tts " + "HTTP/1.1"
        signature_sha = hmac.new(self.APISecret.encode('utf-8'), signature_origin.encode('utf-8'),
                                 digestmod=hashlib.sha256).digest()
        signature_sha = base64.b64encode(signature_sha).decode(encoding='utf-8')
        authorization_origin = "api_key=\"%s\", algorithm=\"%s\", headers=\"%s\", signature=\"%s\"" % (
            self.APIKey, "hmac-sha256", "host date request-line", signature_sha)
        authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')
        v = {
            "authorization": authorization,
            "date": date,
            "host": "ws-api.xfyun.cn"
        }
        url = url + '?' + urlencode(v)
        return url

class TextToSpeech:
    """文本转语音模块 - 单例模式"""
    
    _instance = None
    _lock = threading.Lock()
    
    def __new__(cls):
        """确保单例模式"""
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance
    
    def __init__(self):
        """初始化TTS引擎"""
        if hasattr(self, '_initialized'):
            return
        self._initialized = True
        
        # 讯飞 TTS 配置
        self.APPID = "f6e50ac1"  # 替换为你的 APPID
        self.APIKey = "11ecfe2c70c42cadda1bae3916576ec4"  # 替换为你的 APIKey
        self.APISecret = "NGQwMjdiYTY0ZGZkZDI5MTJlMGQ4Yzhk"  # 替换为你的 APISecret
    
    @classmethod
    def get_instance(cls):
        """获取TTS实例（线程安全）"""
        with cls._lock:
            if cls._instance is None:
                cls._instance = cls()
        return cls._instance
    
    def text_to_wav(self, text, sample_rate=16000):
        """
        将文本转换为WAV音频数据
        
        Args:
            text: 要转换的文本
            sample_rate: 采样率
            
        Returns:
            音频数据的字节序列，如果失败则返回None
        """
        try:
            # 创建讯飞 TTS 参数
            ws_param = Ws_Param(self.APPID, self.APIKey, self.APISecret, text)
            url = ws_param.create_url()
            
            # 创建 WebSocket 连接
            ws = websocket.WebSocketApp(url, on_message=self._on_message, on_error=self._on_error, on_close=self._on_close)
            ws.on_open = self._on_open
            self.audio_data = bytearray()  # 用于存储音频数据
            self.is_closed = False
            self.ws_param = ws_param  # 存储 Ws_Param 实例
            # 启动 WebSocket
            ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
            
            # 等待音频数据收集完成
            while not self.is_closed:
                time.sleep(0.1)
            
            # 将音频数据转换为 WAV 格式
            with io.BytesIO() as wav_buffer:
                with wave.open(wav_buffer, 'wb') as wav_file:
                    wav_file.setnchannels(1)
                    wav_file.setsampwidth(2)
                    wav_file.setframerate(sample_rate)
                    wav_file.writeframes(self.audio_data)
                return wav_buffer.getvalue()
            
        except Exception as e:
            logger.error(f"文本转WAV失败: {e}")
            return None
    
    def text_to_pcm(self, text, sample_rate=16000, channels=1):
        """
        将文本转换为PCM格式的音频数据
        
        Args:
            text: 要转换的文本
            sample_rate: 采样率
            channels: 声道数
            
        Returns:
            PCM格式的音频数据字节序列，如果失败则返回None
        """
        wav_data = self.text_to_wav(text, sample_rate)
        if not wav_data:
            return None
            
        try:
            with wave.open(io.BytesIO(wav_data), 'rb') as wav_file:
                pcm_data = wav_file.readframes(wav_file.getnframes())
                return pcm_data
        except Exception as e:
            logger.error(f"WAV转PCM失败: {e}")
            return None
    
    def _on_open(self, ws):
        """WebSocket 打开回调"""
        def run(*args):
            d = {
                "common": self.ws_param.CommonArgs,
                "business": self.ws_param.BusinessArgs,
                "data": self.ws_param.Data
            }
            d = json.dumps(d)
            ws.send(d)
        
        thread.start_new_thread(run, ())
    
    def _on_message(self, ws, message):
        """WebSocket 消息回调"""
        try:
            message = json.loads(message)
            code = message["code"]
            audio = message["data"]["audio"]
            status = message["data"]["status"]
            
            if status == 2:
                self.is_closed = True
                ws.close()
            
            if code != 0:
                errMsg = message["message"]
                logger.error(f"sid:%s call error:%s code is:%s" % (message["sid"], errMsg, code))
            else:
                audio_bytes = base64.b64decode(audio)
                self.audio_data.extend(audio_bytes)
        
        except Exception as e:
            logger.error(f"接收消息异常: {e}")
    
    def _on_error(self, ws, error):
        """WebSocket 错误回调"""
        logger.error(f"WebSocket 错误: {error}")
        self.is_closed = True
    
    def _on_close(self, ws):
        """WebSocket 关闭回调"""
        logger.info("WebSocket 已关闭")
        self.is_closed = True